"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tsAdd = void 0;
const npm_package_arg_1 = __importDefault(require("npm-package-arg"));
const semver_1 = __importDefault(require("semver"));
const child_process_1 = require("child_process");
const utils_1 = require("./utils");
function makeCommand(dependencies, flags = '') {
    const deps = dependencies
        .map(e => e.match(/[<>]/) ? `"${e}"` : e)
        .join(' ');
    const registryFlag = !utils_1.withFlagCredentials ? '' : (() => {
        const registry = utils_1.getConfig('registry');
        const login = utils_1.getConfig('login');
        const password = utils_1.getConfig('password');
        const { hrefWithCredentials } = utils_1.parseURL(registry, login, password);
        return `--reg=${hrefWithCredentials}`;
    })();
    return `npm i ${flags && `${flags} `}${deps} ${registryFlag}`.trim();
}
const runCommand = (command) => child_process_1.execSync(command, {
    ...!utils_1.getConfig('silent') && { stdio: 'inherit' },
});
async function tsAdd() {
    var _a;
    if (utils_1.getConfig(':make-autocompletion-flags')) {
        const [param] = process.argv.slice(3);
        if (!param.startsWith('-')) {
            return;
        }
        // parsedCLIParams[':make-autocompletion-flags'] = false;
        // parsedCLIParams[':test-flags-run'] = true;
        // await tsAdd();
        // const flags = ObjKeys(flagStack)
        // .map(key => [key, flagAliases[key]])
        const flags = utils_1.ObjEntries(utils_1.flagAliases)
            .map(e => {
            const [flag, ...aliases] = e;
            if (typeof utils_1.flagsWithDefaulValues[flag] === 'string') {
                return [`${flag}=`, aliases.flat().map(e => `${e}=`)];
            }
            return e;
        })
            .flat(2)
            .filter(e => !e.startsWith(':'))
            .map(utils_1.makeFlagWithPrefix);
        const prefixRE = param.startsWith('--') ? /^--/ : /^-[^-]/;
        const processedFlags = flags
            .filter(e => e.match(prefixRE))
            .filter(e => !utils_1.isFlagOrSynonymInCommandLine(e))
            .filter(e => e.startsWith(param));
        const result = processedFlags.length === 1 && ((_a = processedFlags[0]) === null || _a === void 0 ? void 0 : _a.endsWith('='))
            ? [`${processedFlags[0]}1`, `${processedFlags[0]}2`]
            : processedFlags;
        console.info(result.join(' '));
        return;
    }
    if (utils_1.getConfig('completion')) {
        return void utils_1.printMessage.completion();
    }
    if (utils_1.getConfig('version')) {
        return void utils_1.printMessage.version();
    }
    if (utils_1.getConfig('help')) {
        return void utils_1.printMessage.help();
    }
    const { _: packageNames } = utils_1.parsedCLIParams;
    if (!packageNames.length) {
        utils_1.printMessage.noParams();
        if (utils_1.getConfig(':test-flags-run')) {
            return;
        }
        process.exit(1);
    }
    const dependencies = [];
    const devDependencies = [];
    for (const packageName of packageNames) {
        const { name, version, scope } = (() => {
            // Note that some parsePackageName.Result fields of type `string | null` are `string | undefined` actually
            const pkg = npm_package_arg_1.default(packageName);
            return {
                name: pkg.name || '',
                version: pkg.fetchSpec || 'latest',
                scope: pkg.scope || null,
            };
        })();
        let pkg;
        try {
            pkg = await utils_1.fetchPackage(name);
        }
        catch (e) {
            if (e.code === 'E401') {
                utils_1.printMessage.unauthorized();
            }
            else {
                utils_1.printMessage.noPackage(packageName);
            }
            if (utils_1.getConfig(':test-flags-run')) {
                return;
            }
            process.exit(1);
        }
        const { versions, 'dist-tags': tags } = pkg;
        const allVersions = utils_1.ObjKeys(versions);
        const v = tags[version] || version;
        const targetVersion = semver_1.default.maxSatisfying(allVersions, v, { includePrerelease: true });
        if (!targetVersion) {
            utils_1.printMessage.noPackage(packageName);
            if (utils_1.getConfig(':test-flags-run')) {
                return;
            }
            process.exit(1);
        }
        const targetPackage = pkg.versions[targetVersion];
        const hasExplicitTypes = Boolean(targetPackage.types || targetPackage.typings);
        if (!utils_1.getConfig('types-only')) {
            if (targetVersion === tags.latest) {
                const pkgName = utils_1.getConfig('ignore-package-json') ? `${name}@latest` : packageName;
                (utils_1.getConfig('save-dev') ? devDependencies : dependencies).push(pkgName);
            }
            else {
                (utils_1.getConfig('save-dev') ? devDependencies : dependencies).push(packageName);
            }
        }
        if (hasExplicitTypes) {
            continue;
        }
        let typesPackage;
        const typesPackageName = scope
            ? `@types/${scope.slice(1)}__${name.slice(scope.length + 1)}`
            : `@types/${name}`;
        try {
            typesPackage = await utils_1.fetchPackage(typesPackageName);
        }
        catch {
            utils_1.printMessage.noTypesDeclaration(packageName);
            continue;
        }
        const { versions: typeVersions, 'dist-tags': typeTags } = typesPackage;
        const allTypesVersions = utils_1.ObjKeys(typeVersions).filter(v => !typeVersions[v].deprecated);
        if (targetVersion === tags.latest && typeVersions[typeTags.latest].deprecated) {
            continue;
        }
        const typesTargetVersion = targetVersion === tags.latest
            ? typeTags.latest
            : `<=${semver_1.default.major(targetVersion)}.${semver_1.default.minor(targetVersion)}.x`;
        const targetTypesVersion = semver_1.default.maxSatisfying(allTypesVersions, typesTargetVersion, {
            includePrerelease: utils_1.getConfig('allow-prerelease'),
        });
        if (targetTypesVersion) {
            if (targetTypesVersion === typeTags.latest) {
                const tPkgName = `${typesPackageName}${utils_1.getConfig('ignore-package-json') ? '@latest' : ''}`;
                (utils_1.getConfig('save-prod') ? dependencies : devDependencies).push(tPkgName);
            }
            else {
                (utils_1.getConfig('save-prod') ? dependencies : devDependencies).push(`${typesPackageName}@${targetTypesVersion}`);
            }
        }
        else {
            utils_1.printMessage.noTypesDeclaration(packageName);
        }
    }
    utils_1.printMessage.endNoTypesDeclaration();
    let isPrinted = false;
    if (dependencies.length) {
        const command = makeCommand(dependencies, '-P');
        if (!utils_1.getConfig('silent') && !utils_1.getConfig(':test-flags-run')) {
            utils_1.printMessage.command(command);
            isPrinted = true;
        }
        if (!utils_1.getConfig('dry-run') && !utils_1.getConfig(':test-flags-run')) {
            runCommand(command);
        }
    }
    if (devDependencies.length) {
        const command = makeCommand(devDependencies, '-D');
        if (!utils_1.getConfig('silent') && !utils_1.getConfig(':test-flags-run')) {
            utils_1.printMessage.command(command);
            isPrinted = true;
        }
        if (!utils_1.getConfig('dry-run') && !utils_1.getConfig(':test-flags-run')) {
            runCommand(command);
        }
    }
    if (!utils_1.getConfig('silent') && !isPrinted && !utils_1.getConfig(':test-flags-run')) {
        console.info('Nothing to install.');
    }
}
exports.tsAdd = tsAdd;
if (typeof exports.default === 'function') {
    module.exports = exports.default;
    Object.assign(module.exports, exports);
}
